@@ -221,6 +221,10 @@ int ctrl_cmd_handle(struct ctrl_handle *ctrl, struct ctrl_cmd *cmd,
 
 	if (cmd->type == CTRL_TYPE_SET_REPLY ||
 	    cmd->type == CTRL_TYPE_GET_REPLY) {
+		if (ctrl->reply_cb) {
+			ctrl->reply_cb(ctrl, cmd, data);
+			return CTRL_CMD_HANDLED;
+		}
 		if (strncmp(cmd->reply, "OK", 2) == 0) {
 			LOGP(DLCTRL, LOGL_DEBUG, "%s <%s> for %s is OK\n",
 			     get_value_string(ctrl_type_vals, cmd->type),
@@ -510,6 +514,7 @@ struct ctrl_connection *osmo_ctrl_conn_alloc(void *ctx, void *data)
 	INIT_LLIST_HEAD(&ccon->def_cmds);
 
 	ccon->write_queue.bfd.data = data;
+	ccon->write_queue.bfd.fd = -1;
 	ccon->write_queue.write_cb = control_write_cb;
 	ccon->write_queue.read_cb = handle_control_read;
 
